import aflr
from aflr.api_request import APIRequest


class Sound(APIRequest):
    OBJECT_NAME = "sound"

    def __init__(self):
        super().__init__()  # add params to the init performed by the base-class
        # good read is https://stackoverflow.com/questions/1385759/should-init-call-the-parent-classs-init
        self.url = self.api_base + "/sound"
        self.file_url = self.api_base + "/file/sound"
        self.bg_url = self.api_base + "/file/bg"

    def create(self, **params):
        return self._post_request(url=self.url, json=params)

    def retrieve(self, scriptId, parameters={}):
        parameters.update({"scriptId": scriptId})
        return self._get_request(url=self.file_url, request_params=parameters)

    def list(self):
        return self._get_request(url=self.bg_url)

    def download(self, scriptId, parameters={}, destination="."):
        url = self.retrieve(scriptId=scriptId, parameters=parameters).get("url")
        local_filename = self._download_request(url=url, destination=destination)
        return local_filename



import aflr
from aflr.api_request import APIRequest


class Script(APIRequest):
    OBJECT_NAME = "script"

    def __init__(self):
        super().__init__()  # add params to the init performed by the base-class
        # good read is https://stackoverflow.com/questions/1385759/should-init-call-the-parent-classs-init
        self.url = self.api_base + "/script"

    def config_test(self):
        return f"Configured to transact {self.OBJECT_NAME} objects to {self.url} with api_key = {self.api_key}"

    # get scripts list
    def list(self):
        return self._get_request(url=self.url)

    # get script by scriptId
    def retrieve(self, scriptId):
        return self._get_request(url=self.url, path_param=scriptId)

    # create a new script
    def create(self, **params):
        return self._post_request(json=params)



import aflr
import requests
import shutil
import os
import requests
from requests.exceptions import HTTPError


class APIRequest:
    def __init__(self, api_key=None, api_base=None, **params):
        self.api_key = api_key if api_key else aflr.api_key
        self.api_base = api_base if api_base else aflr.api_base
        if self.api_key == None or self.api_key == "your-key":
            self.api_key = os.environ.get("aflr_key", None)
        if self.api_key == None or len(self.api_key) < 30:
            raise ValueError(
                "Please specify a valid api_key or create one here:\nhttps://console.api.audio"
            )
        if not isinstance(self.api_key, str):
            raise TypeError("api_key must be of type string.")

    def _build_header(self):
        # add more headers for analytics in the future
        return {"x-api-key": self.api_key}

    def _post_request(self, json, url=None):
        url = self.url if not url else url
        headers = self._build_header()
        r = requests.post(url=url, headers=headers, json=json)
        self._expanded_raise_for_status(r)
        return r.json()

    def _get_request(self, url, path_param=None, request_params=None):
        headers = self._build_header()  # DRY. To be changed.
        if request_params:
            r = requests.get(url=url, headers=headers, params=request_params)
        elif path_param:
            r = requests.get(url=f"{url}/{path_param}", headers=headers)
        else:
            r = requests.get(url=url, headers=headers)
        self._expanded_raise_for_status(r)
        return r.json()

    def _download_request(self, url, destination):
        local_filename = f"{destination}/{url.split('/')[-1].split('?')[0]}"
        local_filename = local_filename.replace("%243ct10n", "section")
        with requests.get(url, stream=True) as r:
            self._expanded_raise_for_status(r)
            with open(local_filename, "wb") as f:
                shutil.copyfileobj(r.raw, f)
        return local_filename

    def _expanded_raise_for_status(self, res):
        """
        Take a "requests" response object and expand the raise_for_status method to return errors from API
        @param res:
        @return: None
        """
        try:
            res.raise_for_status()
        except HTTPError as e:
            if res.json():
                raise HTTPError('{} \n Error Message from API: \n {}'.format(str(e), res.json()))
            else:
                raise e
        return



from aflr.api_resources.script import Script
from aflr.api_resources.speech import Speech
from aflr.api_resources.voice import Voice
from aflr.api_resources.mastering import Mastering
from aflr.api_resources.sound import Sound



import aflr
from aflr.api_request import APIRequest


class File(APIRequest):
    OBJECT_NAME = "file"

    def __init__(self):
        super().__init__()
        self.url = self.api_base + "/file"

    def config_test(self):
        return f"Configured to transact {self.OBJECT_NAME} objects to {self.url} with api_key = {self.api_key}"

    # get speech files
    # TBD

    # get sound files
    # TBD



import aflr
from aflr.api_request import APIRequest


class Speech(APIRequest):
    OBJECT_NAME = "speech"

    def __init__(self):
        super().__init__()
        self.url = self.api_base + "/speech"
        self.file_url = self.api_base + "/file/speech"

    def config_test(self):
        return f"Configured to transact {self.OBJECT_NAME} objects to {self.url} with api_key = {self.api_key}"

    # get speech url by scriptId
    def retrieve(self, scriptId, section=None, parameters={}):
        params = parameters.copy()
        params.update({"scriptId": scriptId})
        if section:
            params.update({"section": section})
        return self._get_request(
            url=self.file_url,
            request_params=params,
        )

    # create a new text-to-speech
    def create(self, **params):
        # params["api"] = False # To be changed when ms-file is ready
        return self._post_request(json=params)

    # download speech files
    def download(self, scriptId, section=None, parameters={}, destination="."):
        audio_files = self.retrieve(scriptId, section, parameters)
        local_filenames = []
        print(audio_files)
        for key, value in audio_files.items():
            # Review "value"! list of string...
            local_filename = self._download_request(url=value, destination=destination)
            local_filenames.append(local_filename)

        return local_filenames



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="aflr",
    version="0.5.1",
    author="Antonio Tripiana",
    author_email="antonio@aflorithmic.ai",
    description="Python library for the Aflorithmic api.audio API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/aflorithmic/aflr_python",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=['requests >= 2.20; python_version >= "3.0"'],
    python_requires=">=3.6",
)



import aflr
from aflr.api_request import APIRequest


class Voice(APIRequest):
    OBJECT_NAME = "voice"

    def __init__(self):
        super().__init__()
        self.url = self.api_base + "/voice"

    # get voices list
    def list(self, **params):
        return self._get_request(url=self.url, request_params=params)



import aflr
from aflr.api_request import APIRequest


class Mastering(APIRequest):
    OBJECT_NAME = "mastering"

    def __init__(self):
        super().__init__()
        self.url = self.api_base + "/mastering"
        self.file_url = self.api_base + "/file/mastering"

    def config_test(self):
        return f"Configured to transact {self.OBJECT_NAME} objects to {self.url} with api_key = {self.api_key}"

    def create(self, **params):
        return self._post_request(url=self.url, json=params)

    # get mastering file
    def retrieve(self, scriptId, parameters={}):
        params = parameters.copy()
        params.update({"scriptId": scriptId})
        return self._get_request(url=self.file_url, request_params=params)

    # download mastering file
    def download(self, scriptId, parameters={}, destination="."):
        url = self.retrieve(scriptId=scriptId, parameters=parameters).get("url")
        if not url:
            raise Exception(
                "Error: No mastered files found. Please try again in a few seconds."
            )
        local_filename = self._download_request(url=url, destination=destination)
        return local_filename
